import 'package:cpt_rewards/modules/rewards_detail/rewards_detail_page.dart';
import 'package:cs_resources/theme/app_colors_theme.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:auto_route/auto_route.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';
import 'package:router/ext/auto_router_extensions.dart';
import 'package:shared/utils/log_utils.dart';
import 'package:shared/utils/color_utils.dart';
import 'package:widgets/ext/ex_widget.dart';
import 'package:widgets/my_appbar.dart';
import 'package:widgets/my_load_image.dart';
import 'package:widgets/shatter/picker_container.dart';
import 'package:widgets/widget_export.dart';
import 'package:cs_resources/generated/assets.dart';

import '../../../router/page/rewards_page_router.dart';
import './rewards_list_vm.dart';

@RoutePage()
class RewardsListPage extends HookConsumerWidget {
  const RewardsListPage({Key? key}) : super(key: key);

  //启动当前页面
  static void startInstance({BuildContext? context}) {
    if (context != null) {
      context.router.push(const RewardsListPageRoute());
    } else {
      appRouter.push(const RewardsListPageRoute());
    }
  }

  Widget _buildItemLeftSection(BuildContext context, WidgetRef ref, item, _vm) {
    return Column(
      children: [
        Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            const MyAssetImage(
              Assets.rewardsRewardsIndex1,
              // width: 110,
              height: 150,
            ),
            const Text(
              maxLines: 1, // 设置最大行数为2
              overflow: TextOverflow.ellipsis, // 超出部分用省略号表示
              'Cute children multifunctional ……',
              style: TextStyle(
                  fontSize: 16.0,
                  color: Colors.black,
                  fontWeight: FontWeight.w500),
            ),
            Text(
              maxLines: 1, // 设置最大行数为2
              overflow: TextOverflow.ellipsis, // 超出部分用省略号表示
              'Hong Ye Group Co., Ltd',
              style: TextStyle(
                  fontSize: 13.0,
                  color: ColorUtils.string2Color('#808DAF'),
                  fontWeight: FontWeight.w400),
            ).marginOnly(bottom: 5),
            Row(
              children: [
                const Text(
                  '300',
                  style: TextStyle(
                      fontSize: 19.0,
                      color: Colors.black,
                      fontWeight: FontWeight.w500),
                ),
                Text(
                  '350',
                  style: TextStyle(
                      decoration: TextDecoration.lineThrough,
                      decorationColor: ColorUtils.string2Color('#000000'),
                      decorationStyle: TextDecorationStyle.solid,
                      fontSize: 12.0,
                      color: ColorUtils.string2Color('#000000'),
                      fontWeight: FontWeight.w400),
                ).marginOnly(left: 5, right: 5),
                const Text(
                  'Points',
                  style: TextStyle(
                      fontSize: 13.0,
                      color: Colors.black,
                      fontWeight: FontWeight.w400),
                ),
              ],
            ),
          ],
        ).onTap(() {
          RewardsDetailPage.startInstance();
        }),
      ],
    ).paddingOnly(top: 0);
  }

  Widget _buildItemRightSection(
      BuildContext context, WidgetRef ref, item, _vm) {
    return Container(
      // color: Colors.green,
      child: Text(
        item['price'],
        style: const TextStyle(
            fontSize: 14.0,
            color: Colors.black,
            fontWeight: FontWeight.w400), // 设置字体大小
      ),
    );
  }

  // listitem
  Widget _buildSaleItem(BuildContext context, WidgetRef ref, item, _vm) {
    return Container(
      decoration: const BoxDecoration(
        color: Colors.white,
        borderRadius: BorderRadius.all(Radius.circular(6.0)),
        boxShadow: [
          BoxShadow(color: Color.fromRGBO(184, 191, 217, 0.3), blurRadius: 6)
        ],
      ),
      child: Column(
        children: [
          SizedBox(
            width: MediaQuery.of(context).size.width - 30,
            height: 235,
            // margin: const EdgeInsets.only(left: 15, right: 15, top: 12.5),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisAlignment: MainAxisAlignment.start,
              children: [
                _buildItemLeftSection(context, ref, item, _vm)
                // .marginOnly(bottom: 5),
                // _buildItemRightSection(context, ref, item, _vm),
              ],
            ).paddingOnly(left: 20, right: 20),
          ).constrained(
            minHeight: 235,
          ),
        ],
      ),
    ).marginOnly(left: 15, bottom: 15, right: 15);
  }

  // list
  Widget _buildSaleList(BuildContext context, WidgetRef ref, _vm) {
    List itemsList = _vm.state.list.toList();
    return ListView.builder(
      itemCount: itemsList.length,
      itemBuilder: (context, index) {
        return _buildSaleItem(context, ref, itemsList[index], _vm);
      },
    );
  }

  Widget _buildTop(BuildContext context, WidgetRef ref, _vm) {
    final state = ref.watch(rewardsListVmProvider);
    return Container(
      height: 55,
      decoration: const BoxDecoration(
        color: Colors.white,
        borderRadius: BorderRadius.all(Radius.circular(6.0)),
        boxShadow: [
          BoxShadow(color: Color.fromRGBO(184, 191, 217, 0.3), blurRadius: 6)
        ],
      ),
      child: Row(
        children: [
          SizedBox(
              width: MediaQuery.of(context).size.width / 2 - 60,
              child:
                  // Column(
                  //   children: [
                  //     PickerContainer(
                  //       content: state.selectedOption ?? "",
                  //       hint: 'All',
                  //       bgColor: Colors.white,
                  //       margin: const EdgeInsets.only(top: 0),
                  //       onClick: _vm.pickCategory,
                  //     ),
                  //   ],
                  // ),
                  const Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                crossAxisAlignment: CrossAxisAlignment.center,
                children: [
                  Text(
                    'All',
                    style: TextStyle(
                        fontSize: 15.0,
                        color: Colors.black,
                        fontWeight: FontWeight.w500), // 设置字体大小
                  ),
                  MyAssetImage(
                    Assets.rewardsRewardsIconDomn,
                    width: 12,
                    height: 7,
                  ),
                ],
              ).onTap(() {
                // 去详情
                _vm.doDeleteAccountAll(context);
              })).paddingOnly(left: 15, right: 15),
          SizedBox(
              width: MediaQuery.of(context).size.width / 2 - 40,
              child:
                  // Column(
                  //   children: [
                  //     PickerContainer(
                  //       content: state.selectedOption ?? "",
                  //       hint: 'Popularity',
                  //       bgColor: Colors.white,
                  //       margin: const EdgeInsets.only(top: 0),
                  //       onClick: _vm.pickCategory,
                  //     ),
                  //   ],
                  // )
                  const Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                crossAxisAlignment: CrossAxisAlignment.center,
                children: [
                  Text(
                    'Popularity',
                    style: const TextStyle(
                        fontSize: 15.0,
                        color: Colors.black,
                        fontWeight: FontWeight.w500), // 设置字体大小
                  ),
                  MyAssetImage(
                    Assets.rewardsRewardsIconDomn,
                    width: 12,
                    height: 7,
                  ),
                ],
              ).onTap(() {
                // 去详情
                _vm.doDeleteAccount(context);
              })).paddingOnly(left: 15, right: 15)
        ],
      ),
    )
        .marginOnly(top: 12, left: 15, right: 15)
        .backgroundColor(ColorUtils.string2Color('#F2F3F6'));
  }

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final _vm = ref.read(rewardsListVmProvider.notifier);
    final state = ref.watch(rewardsListVmProvider);
    return Scaffold(
      appBar: MyAppBar.appBar(
        context,
        "Recommend",
        backgroundColor: context.appColors.whiteBG,
      ),
      body: Container(
        child: Column(
          children: [
            _buildTop(context, ref, _vm),
            Expanded(
              child: EasyRefresh(
                // 上拉加载
                onLoad: () async {
                  Log.d("----onLoad");
                  _vm.onLoadData();
                },
                // 下拉刷新
                onRefresh: () async {
                  Log.d("----onRefresh");
                  _vm.refreshListData();
                },
                child: Container(
                    color: ColorUtils.string2Color('#F2F3F6'),
                    padding: const EdgeInsets.only(top: 15),
                    child: _buildSaleList(context, ref, _vm)),
              ),
            )
          ],
        ),
      ),
    );
  }
}
